==> SSG-0.2.tar.gz: Period.java <==
// Class Period - An indicator for a period of time;

// Periods can be full years (e.g. "1998"), or parts of a year (e.g. "1998,
// 3rd quarter", "2000 Week 7", etc.).


package sptossg ;

import java.awt.Point;

public class Period   {

    int nyear;
    int nperiod;
    int frequency;

    // Period has four constructors;
    // 1 & 2 - Year only (as string or number) - for annual data
    //    e.g. "1997" or 1997 - annual data for the year 1997
    // 3 - Year & Period as strings
    //    e.g. "1997", "Q3" - 3rd quarter of 1997
    // 4 - year, period, frequency as ints
    //    e.g. 1997,3,4 - also 3rd quarter of 1997

    public Period (String year) {
	this(year, "Y1");
    }

    public Period (int nyear) {
	this(nyear, 1, 1);
    }

    public Period (String year, String period) {
	this.nyear = Integer.parseInt(year);

	Point pt = isPeriodic(period);
	if (pt != null) {
	    nperiod = pt.x;
	    frequency = pt.y;
	} else {
	    // error condition - period string wasn't parseable
	    nperiod = frequency = 0;
	}
    }

    public Period (int nyear, int nperiod, int frequency) {
	this.nyear = nyear;
	this.nperiod = nperiod;
	this.frequency = frequency;
    }

    public int year() {
	return nyear;
    }

    public int period() {
	return nperiod;
    }

    public int frequency() {
	return this.frequency;
    }

    public String toString() {
	if (nyear == 0) {
	    String freqstr = periodicString(frequency);

	    if (nperiod < 0) {
		return "cur - " + (- nperiod) + " " + freqstr + "s";
	    } else
		return "cur + " + nperiod + " " + freqstr + "s";
	} else if (frequency <= 1) {
	    return "" + nyear;
	} else {
	    return (nyear + "/" + periodicString(frequency).substring(0,1) +
		    nperiod);
	}
    }

    // A Period's hashCode is based on it's YEAR & PERIOD components.
    // used by Hashtables for retrievals
    public int hashCode() {
	return 13 * nyear + 17 * nperiod;
    }

    // ****************************************************************
    // boolean comparators

    // equals is needed by Hashtables for retrievals
    public boolean equals(Object p2) {
	if (p2 instanceof Period) {
	    Period p2a = (Period) p2;
	    if (nyear == p2a.nyear && nperiod == p2a.nperiod
		&& frequency == p2a.frequency()) {
		return true;
	    }}
	return false;
    }

    public boolean greater(Period p) {
	if (p == null) return true;
	if (nyear > p.nyear)  return true;
	if (nyear == p.nyear && nperiod > p.nperiod) return true;
	return false;
    }

    public boolean less(Period p) {
	if (p == null) return true;
	if (nyear < p.nyear) return true;
	if (nyear == p.nyear && nperiod < p.nperiod) return true;
	return false;
    }

    // ****************************************************************
    // period parsing

    // The PERIODREFS inner class contains the indicators for parsing
    // periodic references e.q. Q3 <=> 3rd quarter, H2 <=> 2nd half, M6 <=>
    // 12th month, etc.
    static class periodrefs {
	int size;
	String label;
	String letter;
	periodrefs(int sz, String str) {
	    size = sz; label = str;
	    letter = label.substring(0,1);
	};
    };
    static periodrefs reflist[] = {
	new periodrefs(4, "Quarter"),
	new periodrefs(2, "Half"),
	new periodrefs(12, "Month"),
	new periodrefs(365, "Day"),
	new periodrefs(1, "Year")
    };

    static String periodicString(int freq) {
	for (int i = 0; i < reflist.length; i++) {
	    if (freq == reflist[i].size) {
		return reflist[i].label;
	    }
	}
	return "?";
    }

    // isPeriodic - parses a period reference and returns an X,Y point made
    // up as X = period number and Y = the maximum number of periods.
    static public Point isPeriodic(String str) {

	int periodno;
	int periodsize;

	if (str == null || str.equals("ALL") || str.equals("")) {
	  return new Point(1,1);
	};

	for (int i = 0; i < reflist.length; i++) {
	    if  (str.regionMatches(true, 0, reflist[i].letter, 0, 1)) {
		try {
		    periodno =  Integer.parseInt(str.substring(1));
		} catch (Exception exc) {
		    return null;
		}
		periodsize = reflist[i].size;
		if (periodno >= 1 && periodno <= periodsize) {
		    return new Point(periodno, periodsize);
		} else {
		    return null;
		}
	    }
	}
	return null;
    }

    // adjust a Period by an offset, returning a new Period.
    //   e.g. adjust( <1996/Q3>, -5) => <1995/Q2
    public Period adjust(int offset) {
	int per2 = nperiod + offset;
	int yr2 = nyear;

	if (frequency <= 0) { return this; }

	while (per2 <= 0) {
	    per2 += frequency;
	    yr2--;
	}
	while (per2 > frequency) {
	    per2 -= frequency;
	    yr2++;
	}

	Period nper = new Period(yr2, per2, frequency);

	return nper;
    }

    // MAIN - a test routine for period parsing
    public static void main (String args[]) {
	String s1 = "quarter";
	adjusttest();

	Period p1;
	p1 = new Period(0, -3, 4);
	System.out.println("p1 = '" + p1 + "'.");
	p1 = new Period(0, 2, 4);
	System.out.println("p1 = '" + p1 + "'.");

	p1 = new Period(0, -3, 2);
	System.out.println("p1 = '" + p1 + "'.");
	p1 = new Period(0, 2, 2);
	System.out.println("p1 = '" + p1 + "'.");

	p1 = new Period(0, -3, 365);
	System.out.println("p1 = '" + p1 + "'.");
	p1 = new Period(0, 2, 365);
	System.out.println("p1 = '" + p1 + "'.");
    }

    static void adjusttest () {
	// Annual

	System.out.println("Annual test ");

	Period ap1, ap2, ap3 , ap4, ap5, ap6;

	ap1 = new Period("1998");
	System.out.println("ap1 = '" + ap1 + "'.");

	ap2 = ap1.adjust(-1);
	System.out.println("ap2 = '" + ap2 + "'.");
	System.out.println("ap1 = '" + ap1 + "'.");

	ap3 = ap2.adjust(-1);
	System.out.println("ap3 = '" + ap3 + "'.");
	System.out.println("ap2 = '" + ap2 + "'.");

	ap4 = ap1.adjust(2);
	System.out.println("ap4 = '" + ap4 + "'.");
	ap4 = ap4.adjust(3);
	System.out.println("ap4 = '" + ap4 + "'.");

	System.out.println("quarterly test ");

	ap1 = new Period("1998", "Q3");
	System.out.println("ap1 = '" + ap1 + "'.");
	System.out.println("counting down ");
	for (int i = 0; i < 20; i++) {
	    ap2 = ap1.adjust(- i);
	    System.out.println("ap2 = '" + ap2 + "'.");
	}
	System.out.println("counting up ");
	for (int i = 0; i < 20; i++) {
	    ap3 = ap1.adjust(i);
	    System.out.println("ap3 = '" + ap3 + "'.");
	}
    }
}

==> SSG-0.3.tar.gz: Period.java <==
// Class Period - An indicator for a period of time;

// Periods can be full years (e.g. "1998"), or parts of a year (e.g. "1998,
// 3rd quarter", "2000 Week 7", etc.).


package sptossg ;

import java.awt.Point;

public class Period   {

    int nyear;
    int nperiod;
    int frequency;

    // Period has five constructors;
    // 1 - no params - represents 'all periods'.
    // 2 & 3 - Year only (as string or number) - for annual data
    //    e.g. "1997" or 1997 - annual data for the year 1997
    // 4 - Year & Period as strings
    //    e.g. "1997", "Q3" - 3rd quarter of 1997
    // 5 - year, period, frequency as ints
    //    e.g. 1997,3,4 - also 3rd quarter of 1997

    public Period () {
	this(-1,-1,-1);
    }

    public Period (String year) {
	this(year, "Y1");
    }

    public Period (int nyear) {
	this(nyear, 1, 1);
    }

    public Period (String year, String period) {
	try {
	    this.nyear = Integer.parseInt(year);
	} catch (Exception exc) {
	    System.out.println("Period Error: Invalid year " + year);
	    return;
	}

	Point pt = isPeriodic(period);
	if (pt != null) {
	    nperiod = pt.x;
	    frequency = pt.y;
	} else {
	    // error condition - period string wasn't parseable
	    nperiod = frequency = 0;
	}
    }

    public Period (int nyear, int nperiod, int frequency) {
	this.nyear = nyear;
	this.nperiod = nperiod;
	this.frequency = frequency;
    }

    public int year() {
	return nyear;
    }

    public int period() {
	return nperiod;
    }

    public int frequency() {
	return this.frequency;
    }

    public String toString() {
	if (nyear == -1 && nperiod == -1 && frequency == -1) {
	    return "always";
	} else if (nyear == 0) {
	    String freqstr = periodicString(frequency);

	    if (nperiod < 0) {
		return "cur - " + (- nperiod) + " " + freqstr + "s";
	    } else
		return "cur + " + nperiod + " " + freqstr + "s";
	} else if (frequency <= 1) {
	    return "" + nyear;
	} else {
	    return (nyear + "/" + periodicString(frequency).substring(0,1) +
		    nperiod);
	}
    }

    // A Period's hashCode is based on it's YEAR & PERIOD components.
    // used by Hashtables for retrievals
    public int hashCode() {
	return 13 * nyear + 17 * nperiod;
    }

    // ****************************************************************
    // boolean comparators

    // equals is needed by Hashtables for retrievals
    public boolean equals(Object p2) {
	if (p2 instanceof Period) {
	    Period p2a = (Period) p2;
	    if (nyear == p2a.nyear && nperiod == p2a.nperiod
		&& frequency == p2a.frequency()) {
		return true;
	    }}
	return false;
    }

    public boolean greater(Period p) {
	if (p == null) return true;
	if (nyear > p.nyear)  return true;
	if (nyear == p.nyear && nperiod > p.nperiod) return true;
	return false;
    }

    public boolean less(Period p) {
	if (p == null) return true;
	if (nyear < p.nyear) return true;
	if (nyear == p.nyear && nperiod < p.nperiod) return true;
	return false;
    }

    // ****************************************************************
    // period parsing

    // The PERIODREFS inner class contains the indicators for parsing
    // periodic references e.q. Q3 <=> 3rd quarter, H2 <=> 2nd half, M6 <=>
    // 12th month, etc.
    static class periodrefs {
	int size;
	String label;
	String letter;
	periodrefs(int sz, String str) {
	    size = sz; label = str;
	    letter = label.substring(0,1);
	};
    };
    static periodrefs reflist[] = {
	new periodrefs(4, "Quarter"),
	new periodrefs(2, "Half"),
	new periodrefs(12, "Month"),
	new periodrefs(365, "Day"),
	new periodrefs(1, "Year")
    };

    static String periodicString(int freq) {
	if (freq == -1) { return "Static"; }
	for (int i = 0; i < reflist.length; i++) {
	    if (freq == reflist[i].size) {
		return reflist[i].label;
	    }
	}
	return "Unknown " + freq;
    }

    // isPeriodic - parses a period reference and returns an X,Y point made
    // up as X = period number and Y = the maximum number of periods.
    static public Point isPeriodic(String str) {

	int periodno;
	int periodsize;

	if (str == null || str.equals("ALL") || str.equals("")) {
	  return new Point(1,1);
	};

	for (int i = 0; i < reflist.length; i++) {
	    if  (str.regionMatches(true, 0, reflist[i].letter, 0, 1)) {
		try {
		    periodno =  Integer.parseInt(str.substring(1));
		} catch (Exception exc) {
		    return null;
		}
		periodsize = reflist[i].size;
		if (periodno >= 1 && periodno <= periodsize) {
		    return new Point(periodno, periodsize);
		} else {
		    return null;
		}
	    }
	}
	return null;
    }

    // adjust a Period by an offset, returning a new Period.
    //   e.g. adjust( <1996/Q3>, -5) => <1995/Q2
    public Period adjust(int offset) {
	int per2 = nperiod + offset;
	int yr2 = nyear;

	if (frequency <= 0) { return this; }

	while (per2 <= 0) {
	    per2 += frequency;
	    yr2--;
	}
	while (per2 > frequency) {
	    per2 -= frequency;
	    yr2++;
	}

	Period nper = new Period(yr2, per2, frequency);

	return nper;
    }

    // MAIN - a test routine for period parsing
    public static void main (String args[]) {
	String s1 = "quarter";
	adjusttest();

	Period p1;
	p1 = new Period(0, -3, 4);
	System.out.println("p1 = '" + p1 + "'.");
	p1 = new Period(0, 2, 4);
	System.out.println("p1 = '" + p1 + "'.");

	p1 = new Period(0, -3, 2);
	System.out.println("p1 = '" + p1 + "'.");
	p1 = new Period(0, 2, 2);
	System.out.println("p1 = '" + p1 + "'.");

	p1 = new Period(0, -3, 365);
	System.out.println("p1 = '" + p1 + "'.");
	p1 = new Period(0, 2, 365);
	System.out.println("p1 = '" + p1 + "'.");
    }

    static void adjusttest () {
	// Annual

	System.out.println("Annual test ");

	Period ap1, ap2, ap3 , ap4, ap5, ap6;

	ap1 = new Period("1998");
	System.out.println("ap1 = '" + ap1 + "'.");

	ap2 = ap1.adjust(-1);
	System.out.println("ap2 = '" + ap2 + "'.");
	System.out.println("ap1 = '" + ap1 + "'.");

	ap3 = ap2.adjust(-1);
	System.out.println("ap3 = '" + ap3 + "'.");
	System.out.println("ap2 = '" + ap2 + "'.");

	ap4 = ap1.adjust(2);
	System.out.println("ap4 = '" + ap4 + "'.");
	ap4 = ap4.adjust(3);
	System.out.println("ap4 = '" + ap4 + "'.");

	System.out.println("quarterly test ");

	ap1 = new Period("1998", "Q3");
	System.out.println("ap1 = '" + ap1 + "'.");
	System.out.println("counting down ");
	for (int i = 0; i < 20; i++) {
	    ap2 = ap1.adjust(- i);
	    System.out.println("ap2 = '" + ap2 + "'.");
	}
	System.out.println("counting up ");
	for (int i = 0; i < 20; i++) {
	    ap3 = ap1.adjust(i);
	    System.out.println("ap3 = '" + ap3 + "'.");
	}
    }
}
